/****************************************************************************
*
*	Example Code for MNM1221
*
*	Copyright (c) 2006-2023 Panasonic Industry Co., Ltd.
*
*----------------------------------------------------------------------------
*
*	File Name:		mii_register_access_m.c
*	Description:	Example code to access PHY registers in MASTER mode
*   Notes:
*	- This code is just for reference, so that Panasonic does NOT assume 
*	  responsibility or liability for actual use based upon this code.
*	- This file is NOT for warranty of correct behavior, 
*	  and might include bugs, so that you have to be careful.
*	- The codes are very basic only, therefore you will need additional code 
*	  such as the process at the occurrence of errors.
*	- It is assumed that TAB of your PC is set to 4space width.
*
*----------------------------------------------------------------------------
*	Revision:	Date(y/m/d):	Author:			Comments:
*	0.0			2006/07/13		Y. Fujisaki		Initial release
*	0.1			2007/11/06		Y. Fujisaki		Cleanup
*	0.2			2009/09/09		Y. Fujisaki		Added KSZ8041
*	0.3			2010/09/14		Y. Fujisaki		Added wait after writing
*	1.0			2012/02/10		Y. Fujisaki		Cleanup
*	1.1			2018/11/08		Y. Fujisaki		Added KSZ8081
*	1.2			2023/03/31		Y. Fujisaki		Added DP83822 and removed 
*												register definitions for MDIO 
*
*****************************************************************************/

#include	"mnm1221_m.h"	/* Modify path according to your file structure */


/****************************************************************************/
/* Definitions and Declarations												*/
/****************************************************************************/

/*--------------------------------------------------------------------------*/
/* PHY register definition													*/
/*--------------------------------------------------------------------------*/
#define	PHY_ADDR				0x01	/* PHY Address by pin configuration */

/* KSZ8001 */
#define	KSZ8001_PHY_CTRL		0x1e	/* PHY Control Register */
#define	KSZ8001_100TX_CTRL		0x1f	/* 100BASE-TX PHY Control Register */

/* KSZ8041 */
#define	KSZ8041_BASIC_CTRL		0x00	/* Basic Control Register */
#define	KSZ8041_PHY_CTRL_1		0x1e	/* PHY Control 1 Register */
#define	KSZ8041_PHY_CTRL_2		0x1f	/* PHY Control 2 Register */

/* KSZ8081 */
#define	KSZ8081_BASIC_CTRL		KSZ8041_BASIC_CTRL	/* the same as KSZ8041 */
#define	KSZ8081_PHY_CTRL_2		KSZ8041_PHY_CTRL_2	/* the same as KSZ8041 */

/* DP83822 */
#define DP83822_BASIC_CTRL		0x00	/* Basic Control Register */


/*--------------------------------------------------------------------------*/
/* Declaration (prototype) of functions										*/
/*--------------------------------------------------------------------------*/
extern short init_phy_ksz8001(void);
extern short init_phy_ksz8041(void);
extern short init_phy_ksz8081(void);
extern short init_phy_dp83822(void);


/* Provide the following function by yourself. */
extern void wait_5us(void);				/* waiting 5us or more */



/****************************************************************************/
/* Global Functions															*/
/****************************************************************************/

/*----------------------------------------------------------------------------
*
*	Function Name:	init_phy_ksz8001
*
*	Description:	initialize registers of KSZ8001 (Microchip)
*
*	Argument(s):	none
*
*	Return Value:	0: OK, -1: time-out error
*
*	Comment:		Call after releasing reset of PHY.
*					Adjust time-out setting according to your CPU speed.
*
*---------------------------------------------------------------------------*/
short init_phy_ksz8001(void)
{
	unsigned short i;

	/* set LED mode to 01 (LED0: LINK) */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8001_PHY_CTRL;
	MNM1221_M_MDIO_WDAT = 0x4000;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	/* disable auto MDI/MDIX */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8001_100TX_CTRL;
	MNM1221_M_MDIO_WDAT = 0xa500;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	init_phy_ksz8041
*
*	Description:	initialize registers of KSZ8041 (Microchip)
*
*	Argument(s):	none
*
*	Return Value:	0: OK, -1: time-out error
*
*	Comment:		Call after releasing reset of PHY.
*					Adjust time-out setting according to your CPU speed.
*
*---------------------------------------------------------------------------*/
short init_phy_ksz8041(void)
{
	unsigned short i;

	/* disable auto-negotiation */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8041_BASIC_CTRL;
	MNM1221_M_MDIO_WDAT = 0x2100;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	/* set LED mode to 01 (LED0: LINK) */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8041_PHY_CTRL_1;
	MNM1221_M_MDIO_WDAT = 0x4000;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	/* disable auto MDI/MDIX */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8041_PHY_CTRL_2;
	MNM1221_M_MDIO_WDAT = 0xa100;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	init_phy_ksz8081
*
*	Description:	initialize registers of KSZ8081 (Microchip)
*
*	Argument(s):	none
*
*	Return Value:	0: OK, -1: time-out error
*
*	Comment:		Call after releasing reset of PHY.
*					Adjust time-out setting according to your CPU speed.
*
*---------------------------------------------------------------------------*/
short init_phy_ksz8081(void)
{
	unsigned short i;

	/* disable auto-negotiation */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8081_BASIC_CTRL;
	MNM1221_M_MDIO_WDAT = 0x2100;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	/* disable auto MDI/MDIX and set LED0:"LINK" */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | KSZ8081_PHY_CTRL_2;
	MNM1221_M_MDIO_WDAT = 0xa110;
	MNM1221_M_MDIO_WR = 1;

	/* waiting for WR complete */
	/* Adjust time-out setting according to your CPU speed. */
	for (i = 0; MNM1221_M_MDIO_WR == 1; i++) {
		if (i >= 0xffff) {
			return -1;					/* time-out error */
		}
	}
	wait_5us();

	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	init_phy_dp83822
*
*	Description:	initialize registers of DP83822 (TI)
*
*	Argument(s):	none
*
*	Return Value:	0
*
*	Comment:		Call after releasing reset of PHY.
*
*---------------------------------------------------------------------------*/
short init_phy_dp83822(void)
{
	/* 100 Mbps, Disable auto-negotiation, Full-duplex */
	MNM1221_M_MDIO_ADDR = (PHY_ADDR << 5) | DP83822_BASIC_CTRL;
	MNM1221_M_MDIO_WDAT = 0x2100;
	MNM1221_M_MDIO_WR = 1;

	return 0;
}


/***************************************** End of mii_register_access_m.c ***/
