/****************************************************************************
*
*	Example Code for MNM1221
*
*	Copyright (c) 2004-2023 Panasonic Industry Co., Ltd.
*
*----------------------------------------------------------------------------
*
*	File Name:		mnm1221_m.c
*	Description:	Example code to access MNM1221 as MASTER mode
*   Notes:
*	- This code is just for reference, so that Panasonic does NOT assume 
*	  responsibility or liability for actual use based upon this code.
*	- This file is NOT for warranty of correct behavior, 
*	  and might include bugs, so that you have to be careful.
*	- The codes are very basic only, therefore you will need additional code 
*	  such as the process at the occurrence of errors.
*	- It is assumed that TAB of your PC is set to 4space width.
*
*----------------------------------------------------------------------------
*	Revision:	Date(y/m/d):	Author:			Comments:
*	0.0			2004/02/16		Y. Fujisaki		Initial release
*	0.1			2004/03/01		Y. Fujisaki		Cleanup
*	0.2			2004/05/20		Y. Fujisaki		Minor edits
*	0.3			2023/03/31		Y. Fujisaki		Minor edits
*
*****************************************************************************/

#include	"mnm1221_m.h"	/* Modify path according to your file structure */


/****************************************************************************/
/* Definitions and Declarations												*/
/****************************************************************************/
/*
* Some of the followings accessed in also other files should be moved to
* the header file you provide separately.
*/

/*--------------------------------------------------------------------------*/
/* Definition of constants													*/
/*--------------------------------------------------------------------------*/
#define	MAX_NS				MAX_N_SLAVE_NODE

#ifdef MASTER_16BIT_ACCESS	/* 16bits bus width */
	#define	N_DATA			WORD_SIZE_OF_BLOCK
#else						/* 32bits bus width */
	#define	N_DATA			DWORD_SIZE_OF_BLOCK
#endif

#define	RT_FRAME_FORM		3			/* Do not change this value. */
#define N_WDT_RATE			5			/* setting for timeout rate */
/*
* Consider the value about timeout according to your system.
* For safety, large value is not recomended.
*/

/* phase to control MNM1221 --> see ctrl_mnm1221_m() */
enum {
	PH_RESET = -1,
	PH_INIT = 0,
	PH_WAITING,
	PH_PREPARE,
	PH_START,
	PH_RUNNING
};

/*--------------------------------------------------------------------------*/
/* Definition of bit field													*/
/*--------------------------------------------------------------------------*/
/* error code in initializing */
#define	MSK_ERR_MACID		MSK_MACID	/* error MACID */
#define B_MIS_N_BLK			0x0040		/* mis-set N of block */
#define B_MIS_MODE			0x0020		/* mis-set MODE */
#define B_NON_MACID			0x0010		/* not found MACID */
#define B_DUP_MACID_B		0x0008		/* duplicate MACID for base */
#define B_DUP_MACID_A		0x0004		/* duplicate MACID for actual */
#define B_MIS_NODE_SUM_B	0x0002		/* mis-set NODE sum for base */
#define B_MIS_NODE_SUM_A	0x0001		/* mis-set NODE sum for actual */

/* error code in running */
#define	B_TIMEOUT			0x0001		/* timeout */


/*--------------------------------------------------------------------------*/
/* Declaration of typedef													*/
/*--------------------------------------------------------------------------*/
/* communication data */
typedef struct {
#ifdef MASTER_16BIT_ACCESS
	/* 16bits bus width */
	/* An element of the array is 16bit data. */
	unsigned short data[WORD_SIZE_OF_BLOCK];
#else
	/* 32bits bus width */
	/* An element of the array is 32bit data. */
	unsigned long data[DWORD_SIZE_OF_BLOCK];
#endif
} Com_buf;

/* error status */
typedef struct {
	unsigned short init;		/* error code in initializing */
	unsigned short run;			/* error code in running */
	unsigned short count;		/* CRC error count */
} Com_err;

/* copy of MNM1221 status register */
typedef struct {
	unsigned long data_crc_err;
	unsigned short state;
	unsigned short err_flgs1;
	unsigned short err_flgs2;
	unsigned short node_sum;	/* sum of slave nodes */
	unsigned short blk_sum;		/* sum of slave blocks */
} Mnm1221;


/*--------------------------------------------------------------------------*/
/* Declaration and definition of variables used in also other files			*/
/*--------------------------------------------------------------------------*/
/*
* If your compiler does not allow that a file includes both declaration and 
* definition of variables, the declaration should be moved to the other file.
*/

/* declaration */
extern Com_buf tx_buf[];		/* TX data buffer */
extern Com_buf rx_buf[];		/* RX data buffer */
extern Com_err com_err;			/* error status */
extern Mnm1221 mnm1221;			/* MNM1221 status */

/* definition */

/* Via the following buffer, the exchange of communication data should be done. */
Com_buf tx_buf[MAX_NS];			/* TX data buffer */
Com_buf rx_buf[MAX_NS];			/* RX data buffer */

Com_err com_err;				/* error status */
Mnm1221 mnm1221;				/* MNM1221 status */


/*--------------------------------------------------------------------------*/
/* Definition of local variables											*/
/*--------------------------------------------------------------------------*/

/* reference table to access communication buffer */
/*   one node, which contains one or more blocks, per one element of the array */
/*   The following two data have the same bit field, but order is different. */
/*     free order, corresponding with tx_buf[], rx_buf[] */
static unsigned short slave_inf_base[MAX_NS];
/*     connection order, corresponding with memory in MNM1221 */
static unsigned short slave_inf_actual[MAX_NS];

/* reference table to arrange to the order of memory in MNM1221 */
/*   one block per one element of the array */
static short order_ref_tbl[MAX_NS];	

/* watchdog timer to detect timeout */
volatile static unsigned short watchdog_tim;


/*--------------------------------------------------------------------------*/
/* Declaration (prototype) of functions										*/
/*--------------------------------------------------------------------------*/
/* global (called in other files) */
extern void int_rx_mnm1221_m(void);		/* use in INTRX interrupt */
extern short ctrl_mnm1221_m(void);		/* use in timer interrupt for NC */

/* local */
static void init_mnm1221_m(void);
static short init_ref_table(void);
static void clr_com_buf(void);
static void set_slave_inf_base(void);
static void get_slave_inf_actual(void);
static short chk_slave_node_sum(void);
static short valid_sum(unsigned short *p);
static short chk_dupl_macid(unsigned short *slave_inf, short n, short err_flg);
static short make_order_tbl(void);
static void set_buf_order(short *p, short order, short n);
static void clr_mnm1221_tx_mem(void);
static void xchg_com_data(void);
static short is_timeout(void);

/* local and general */
static void mem_copy_l(volatile unsigned long *ptr1, volatile unsigned long *ptr2, short n);
static void mem_copy_s(volatile unsigned short *ptr1, volatile unsigned short *ptr2, short n);

/* for test only */
static void set_txbuf_example(short cmd_code);
/*--------------------------------------------------------------------------*/


/****************************************************************************/
/* Global Functions															*/
/****************************************************************************/

/*----------------------------------------------------------------------------
*
*	Function Name:	int_rx_mnm1221_m
*
*	Description:	interrupt routin by INTRX signal from MNM1221
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:		Modify to interrupt function, 
*					or call in the interrupt function you provide.
*
*---------------------------------------------------------------------------*/
void int_rx_mnm1221_m(void)
{
#ifdef MASTER_16BIT_ACCESS		/* 16bits bus */
	unsigned long ul_temp;
#endif

	mnm1221.state = MNM1221_M_STATE;

	/* If not RUNNING state (cyclic transmission), return. */
	if (!(mnm1221.state & B_RUNNING)) {
		mnm1221.err_flgs1 = MNM1221_M_ERR_FLAGS1;
		return;
	}

	mnm1221.err_flgs2 = MNM1221_M_ERR_FLAGS2;
/*
* Normally, err_flgs1 and 2 are not used. They are monitored only.
* Because MNM1221 has error recovering function in RINNING state, 
* the received data can be used as long as data_crc_err indicates OK.
*/

#ifdef MASTER_16BIT_ACCESS		/* 16bits bus */
	ul_temp = MNM1221_M_DCRC_ERR_H;
	ul_temp <<= 16;
	ul_temp |= MNM1221_M_DCRC_ERR_L;
	mnm1221.data_crc_err = ul_temp;
#else							/* 32bits bus */
	mnm1221.data_crc_err = MNM1221_M_DCRC_ERR;
#endif

	/* To read valid data only, if error, do not switch RX memory bank.*/
	if (mnm1221.data_crc_err) {
	/* error detected */
		com_err.count++;
		/* If necessary, add "continuous CRC error" detection. */

	} else {
	/* data is OK */
		MNM1221_M_RXMEM_HOLD = 0;	/* switch bank */
		MNM1221_M_RXMEM_HOLD = 1;	/* return hold state */
	}

	watchdog_tim = 0;		/* clear watchdog timer */
}


/*----------------------------------------------------------------------------
*
*	Function Name:	ctrl_mnm1221_m
*
*	Description:	control MNM1221 as master mode
*
*	Argument(s):	none
*
*	Return Value:	the next phase
*
*	Comment:		It is assumed that this function is called at the beginning 
*					of the timer interrupt for NC calculation (e.g. T = 1ms).
*					If this function is placed in main loop, the data exchanging 
*					and timeout checking in running (cyclic transmission) state 
*					have to be moved to another timer interrupt.
*
*					An example to set communication data is as follows:
*
*						tx_buf[0].data[1] = 2;
*						2 as data value is set into data#1 in block#0 of tx_buf
*
*					For debug, the return value should be monitored.
*
*---------------------------------------------------------------------------*/
short ctrl_mnm1221_m(void)
{
/* Select either of the followings according to your initializing process. */
#if 0
	static short phase = 0;
#else
	static short phase;		/* need RAM clear after reset-release separately */
#endif

	switch (phase) {
	/*--- initializing -----------------------------------------------------*/
	case PH_INIT:					/* MNM1221 is in INITIAL state. */
		phase = PH_WAITING;

		com_err.init = 0;
		com_err.run = 0;
		com_err.count = 0;

		mnm1221.state = MNM1221_M_STATE;
		init_mnm1221_m();
		MNM1221_M_INITF_TX = 1;		/* transmit initial frame */
		break;

	/*--- waiting for ready state ------------------------------------------*/
	case PH_WAITING:				/* MNM1221 is in RING-CONFIG state. */
		/* Until all Slaves are established, Master will wait in this phase. */
		/* If necessary, add timeout detection dedicated to RING-CONFIG state. */
		if (mnm1221.state & B_READY) {
			phase = PH_PREPARE;
		}
		break;

	/*--- check slave information and make reference table -----------------*/
	case PH_PREPARE:				/* MNM1221 is in READY state. */
		if (com_err.init = init_ref_table()) {
			/* Add error routine. */
#if 0
			phase = PH_RESET;		/* depend on your application */
#endif
		} else {
			phase = PH_START;
		}
		break;

	/*--- start of cyclic transmission -------------------------------------*/
	case PH_START:					/* MNM1221 is in READY state. */
		phase = PH_RUNNING;
		clr_mnm1221_tx_mem();		/* clear TX memory, just in case */
		/* This clearing is unnecessary if initial data setting is ensured. */

/************************************************************************/
/* This is for test. You must replace with your application. 			*/
/*----------------------------------------------------------------------*/
		set_txbuf_example(0x00);	/* NOP as initial TX data */
		xchg_com_data();			/* exchange communication data */
/******************************************************** end of test ***/

		watchdog_tim = 0;			/* clear watchdog timer */
		MNM1221_M_CYCL_START = 1;	/* start cyclic transmission */
		break;

	/*--- running state (cyclic transmission) ------------------------------*/
	case PH_RUNNING:				/* MNM1221 is in RUNNING state. */

/************************************************************************/
/* This is for test. You must replace with your application. 			*/
/*----------------------------------------------------------------------*/
/*
* In actual application, the routin setting to TX buffer will be placed 
* after NC calculation. i.e. The routin is not in ctrl_mnm1221_m(), but 
* at the end of the timer interrupt for NC calculation.
*/
		set_txbuf_example(0x20);	/* Position command */
/******************************************************** end of test ***/

		xchg_com_data();			/* exchange communication data */
		if (is_timeout()) {
			com_err.run |= B_TIMEOUT;
			/* Add error routine. */
#if 0
			phase = PH_RESET;		/* depend on your application */
#endif
		}
		break;

	/*--- reset ------------------------------------------------------------*/
	case PH_RESET:
	default:
		phase = PH_INIT;
		MNM1221_M_RESET = 1;		/* reset MNM1221 */
		break;
	}

	return phase;
}


/****************************************************************************/
/* Local Functions															*/
/****************************************************************************/

/*----------------------------------------------------------------------------
*
*	Function Name:	init_mnm1221_m
*
*	Description:	initialize registers of MNM1221
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static void init_mnm1221_m(void)
{
	MNM1221_M_RTF_FORM = RT_FRAME_FORM;
	MNM1221_M_ERR_COUNT = 0;	/* not error count */
	MNM1221_M_TXTIM_SEL = 1;	/* external period timer  */
	MNM1221_M_RXMEM_HOLD = 1;	/* hold RX buffer memory */

	MNM1221_M_INIT_DONE = 1;	/* initialize is done */
}


/*----------------------------------------------------------------------------
*
*	Function Name:	init_ref_table
*
*	Description:	initialize reference table to access memory in MNM1221
*
*	Argument(s):	none
*
*	Return Value:	0: OK, others: error
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static short init_ref_table(void)
{
	short err_code;

	clr_com_buf();
	set_slave_inf_base();
	get_slave_inf_actual();

	if (err_code = chk_slave_node_sum()) {
		return err_code;
	}
	if (err_code = chk_dupl_macid(slave_inf_actual, mnm1221.node_sum, B_DUP_MACID_A)) {
		return err_code;
	}
	if (err_code = chk_dupl_macid(slave_inf_base, mnm1221.node_sum, B_DUP_MACID_B)) {
		return err_code;
	}
	if (err_code = make_order_tbl()) {
		return err_code;
	}
	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	clr_com_buf
*
*	Description:	clear communication buffer memory
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:		the cleared buffer is not in MNM1221, but in CPU
*
*---------------------------------------------------------------------------*/
static void clr_com_buf(void)
{
	short i, j;

	/* TX buffer */
	for (i = 0; i < MAX_NS; i++) {
		for (j = 0; j < N_DATA; j++) {
			tx_buf[i].data[j] = 0;
		}
	}

	/* RX buffer */
	for (i = 0; i < MAX_NS; i++) {
		for (j = 0; j < N_DATA; j++) {
			rx_buf[i].data[j] = 0;
		}
	}
}


/*----------------------------------------------------------------------------
*
*	Function Name:	set_slave_inf_base
*
*	Description:	setting slave information table as base
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:		The table is free order, but closed upper
*					i.e. invalid between valid data is not permitted.
*					Also, the table does not need to be arranged for MAC-ID order.
*					This function must be modified to your configuration style.
*
*---------------------------------------------------------------------------*/
static void set_slave_inf_base(void)
{
	short i;
/*
* The following is just for test, 
* so should be replace with setting based on the user interface for configuration.
*/
	/* e.g. 4 Servo slaves */
	/*--------------------------------------------------------------*/
	/*                   active      mode        MACID     block N  */
	/*--------------------------------------------------------------*/
	slave_inf_base[0] = (1 << 15) | (1 << 13) | (1 << 8) | 1;
	slave_inf_base[1] = (1 << 15) | (1 << 13) | (2 << 8) | 1;
	slave_inf_base[2] = (1 << 15) | (1 << 13) | (3 << 8) | 1;
	slave_inf_base[3] = (1 << 15) | (1 << 13) | (4 << 8) | 1;
	/*--------------------------------------------------------------*/

	/* not presence (i.e. inactive, invalid) */
	/* MSB (active bit) must be set to zero. */
	for (i = 4; i < MAX_NS; i++) {
		slave_inf_base[i] = (0 << 15) | (1 << 13) | (31 << 8) | 1;
	}
}


/*----------------------------------------------------------------------------
*
*	Function Name:	get_slave_inf_actual
*
*	Description:	get slave informations from MNM1221
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static void get_slave_inf_actual(void)
{
#ifdef MASTER_16BIT_ACCESS		/* 16bits bus */

	mnm1221.node_sum = (short)MNM1221_M_NODE_SUM;
	mnm1221.blk_sum = (short)MNM1221_M_BLK_SUM;

	mem_copy_s(slave_inf_actual, P_MNM1221_M_SINF_BGN, MAX_NS);

#else							/* 32bits bus */
	short i;
	long l_temp;
	volatile unsigned long *p;

	mnm1221.node_sum = (short)MNM1221_M_NODE_SUM;
	mnm1221.blk_sum = (short)MNM1221_M_BLK_SUM;

	p = P_MNM1221_M_SINF_BGN;
	for (i = 0; i < (MAX_NS / 2); i++) {
		l_temp = *p++;
		slave_inf_actual[i * 2] = (short)l_temp;
		slave_inf_actual[(i * 2) + 1] = (short)(l_temp >> 16);
	}

#endif
}


/*----------------------------------------------------------------------------
*
*	Function Name:	chk_slave_node_sum
*
*	Description:	check mismatch of slave node sum
*
*	Argument(s):	none
*
*	Return Value:	0: OK, others: error
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static short chk_slave_node_sum(void)
{
	if (valid_sum(slave_inf_actual) != mnm1221.node_sum) {
		return B_MIS_NODE_SUM_A;
	} else if (valid_sum(slave_inf_base) != mnm1221.node_sum) {
		return B_MIS_NODE_SUM_B;
	} else {
		return 0;
	}
}


/*----------------------------------------------------------------------------
*
*	Function Name:	valid_sum
*
*	Description:	search valid data in slave information table
*
*	Argument(s):	pointer of slave information table
*
*	Return Value:	the total number of valid data
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static short valid_sum(unsigned short *p)
{
	short i;
	short n;

	n = 0;
	for (i = 0; i < MAX_NS; i++) {
		if (*p++ & B_ACTIVE) {
			n++;
		}
	}
	return n;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	chk_dupl_macid
*
*	Description:	duplicate MACID(slave node address) check
*
*	Argument(s):	*slave_inf ... pointer for slave information table
*					n          ... number of checking
*					err_flg    ... error flag at detected error
*
*	Return Value:	0: OK, others: error
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static short chk_dupl_macid(unsigned short *slave_inf, short n, short err_flg)
{
	short i, j;
	short macid;

	for (i = 0; i < n; i++) {
		for (j = 0; j < n; j++) {
			if (i == j) {
				continue;
			}
			macid = *(slave_inf + i) & MSK_MACID;
			if (macid == (*(slave_inf + j) & MSK_MACID)) {
				return (macid | err_flg);
			}
		}
	}
	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	make_order_tbl
*
*	Description:	make order reference table
*
*	Argument(s):	none
*
*	Return Value:	0: OK, others: error
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static short make_order_tbl(void)
{
	short i, j, k;
	short blk_order;
	short found;
	short base_macid;
	short base_mode;
	short base_nblk;
	short actual;

	k = 0;
	for (i = 0; i < mnm1221.node_sum; i++) {
		blk_order = 0;
		base_macid = slave_inf_base[i] & MSK_MACID;
		base_mode = slave_inf_base[i] & MSK_MODE;
		base_nblk = slave_inf_base[i] & MSK_N_BLK;

		/* scan to search the data containing the same MACID */
		for (j = 0; j < mnm1221.node_sum; j++) {
			actual = slave_inf_actual[j];

			if (base_macid == (actual & MSK_MACID)) {
			/* hit MACID*/
				/* check MODE */
				if (base_mode != (actual & MSK_MODE)) {
					return (base_macid | B_MIS_MODE);	/* different MODE */
				}
				/* check N */
				if (base_nblk != (actual & MSK_N_BLK)) {
					return (base_macid | B_MIS_N_BLK);	/* different N */
				}
				/* data OK --> set order into the table */
				set_buf_order(&order_ref_tbl[k], blk_order, base_nblk);
				k += base_nblk;
				found = 1;
				break;

			} else {
			/* not hit */
				blk_order += (actual & MSK_N_BLK);
				found = 0;
			}

		}
		if (!found) {
			return (base_macid | B_NON_MACID);		/* not found MACID */
		}
	}

	return 0;
}


/*----------------------------------------------------------------------------
*
*	Function Name:	set_buf_order
*
*	Description:	set the order of buffer in MNM1221 into the table
*
*	Argument(s):	p     ... pointer for order_ref_tbl
*					order ... order of the block
*					n     ... number of the blocks
*
*	Return Value:	none
*
*	Comment:
*
*---------------------------------------------------------------------------*/
static void set_buf_order(short *p, short order, short n)
{
	for (; n > 0; n--) {
		*p++ = order++;
	}
}


/*----------------------------------------------------------------------------
*
*	Function Name:	clr_mnm1221_tx_mem
*
*	Description:	clear TX memory in MNM1221
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:		
*
*---------------------------------------------------------------------------*/
static void clr_mnm1221_tx_mem(void)
{
	short i;

#ifdef MASTER_16BIT_ACCESS			/* 16bits bus */
	volatile unsigned short *p;
#else								/* 32bits bus */
	volatile unsigned long *p;
#endif

/* one bank */
	p = P_MNM1221_TX_MEM_BGN;
	for (i = (mnm1221.blk_sum * N_DATA); i > 0; i--) {
		*p++ = 0;
	}
	MNM1221_M_TXMEM_SW = 1;			/* switch bank  */

/* the other bank */
	p = P_MNM1221_TX_MEM_BGN;
	for (i = (mnm1221.blk_sum * N_DATA); i > 0; i--) {
		*p++ = 0;
	}
	MNM1221_M_TXMEM_SW = 1;			/* switch bank  */
}


/*----------------------------------------------------------------------------
*
*	Function Name:	xchg_com_data
*
*	Description:	exchange communication buffer data between CPU and MNM1221
*
*	Argument(s):	none
*
*	Return Value:	none
*
*	Comment:		In the exchange, the buffer order is arranged.
*
*---------------------------------------------------------------------------*/
static void xchg_com_data(void)
{
#ifdef MASTER_16BIT_ACCESS	/* 16bits bus width */

	short i;
	volatile unsigned short *p;

	/* TX */
	for (i = 0; i < mnm1221.blk_sum; i++) {
		p = P_MNM1221_TX_MEM_BGN;
		p += (order_ref_tbl[i] * N_DATA);
		mem_copy_s(p, &(tx_buf[i].data[0]), N_DATA);
	}
	MNM1221_M_TXMEM_SW = 1;			/* switch TX bank  */

	/* RX */
	for (i = 0; i < mnm1221.blk_sum; i++) {
		p = P_MNM1221_RX_MEM_BGN;
		p += (order_ref_tbl[i] * N_DATA);
		mem_copy_s(&(rx_buf[i].data[0]), p, N_DATA);
	}

#else						/* 32bits bus width */

	short i;
	volatile unsigned long *p;

	/* TX */
	for (i = 0; i < mnm1221.blk_sum; i++) {
		p = P_MNM1221_TX_MEM_BGN;
		p += (order_ref_tbl[i] * N_DATA);
		mem_copy_l(p, &(tx_buf[i].data[0]), N_DATA);
	}
	MNM1221_M_TXMEM_SW = 1;			/* switch TX bank  */

	/* RX */
	for (i = 0; i < mnm1221.blk_sum; i++) {
		p = P_MNM1221_RX_MEM_BGN;
		p += (order_ref_tbl[i] * N_DATA);
		mem_copy_l(&(rx_buf[i].data[0]), p, N_DATA);
	}

#endif
}


/*----------------------------------------------------------------------------
*
*	Function Name:	is_timeout
*
*	Description:	timeout check
*
*	Argument(s):	none
*
*	Return Value:	0: OK, -1: timeout
*
*	Comment:		using watchdog_tim
*
*---------------------------------------------------------------------------*/
static short is_timeout(void)
{
	short i;

	/* Here, insert "Disable Interrupt". */
	if (++watchdog_tim > N_WDT_RATE) {
		watchdog_tim = N_WDT_RATE;
		i = -1;
	} else {
		i = 0;
	}
	/* Here, insert "Enable Interrupt". */

	return i;
}


/****************************************************************************/
/* General Functions														*/
/****************************************************************************/

/*----------------------------------------------------------------------------
*
*	Function Name:	mem_copy_l
*
*	Description:	copy from memory 2 to memory 1
*
*	Argument(s):	ptr1 ... pointer of memory 1
*					ptr2 ... pointer of memory 2
*					n    ... number of copying
*
*	Return Value:	none
*
*	Comment:		32bits access
*
*---------------------------------------------------------------------------*/
static void mem_copy_l(volatile unsigned long *ptr1, volatile unsigned long *ptr2, short n)
{
	for (; n > 0; n--) {
		*ptr1++ = *ptr2++;
	}
}


/*----------------------------------------------------------------------------
*
*	Function Name:	mem_copy_s
*
*	Description:	copy from memory 2 to memory 1
*
*	Argument(s):	ptr1 ... pointer of memory 1
*					ptr2 ... pointer of memory 2
*					n    ... number of copying
*
*	Return Value:	none
*
*	Comment:		16bits access
*
*---------------------------------------------------------------------------*/
static void mem_copy_s(volatile unsigned short *ptr1, volatile unsigned short *ptr2, short n)
{
	for (; n > 0; n--) {
		*ptr1++ = *ptr2++;
	}
}


/****************************************************************************/
/* Test Functions															*/
/****************************************************************************/

/*----------------------------------------------------------------------------
*
*	Function Name:	set_txbuf_example
*
*	Description:	an example for test to set the command data to tx_buf[]
*
*	Argument(s):	cmd_code ... Command Code in Command data block
*					Only at start-up, value 0 i.e. NOP command is allowed.
*					After using the other command, NOP must not be used.
*
*	Return Value:	none
*
*	Comment:		This function is for test only.
*					It is assumed that each Slave has one block only 
*					and all slaves are given the same command code.
*
*---------------------------------------------------------------------------*/
static void set_txbuf_example(short cmd_code)
{
	static short update_counter;
	short i;
	short ud_ctr;
	short macid;

	if (cmd_code == 0) {		/* NOP command at start */
		update_counter = 0;
	} else {					/* valid command */
		update_counter++;
	}

	ud_ctr = (update_counter & 0x0003) << 5;
	cmd_code = (cmd_code & 0x007f) << 8;

	/* It is assumed that the number of nodes is the same as blocks. */
	for (i = 0; i < mnm1221.blk_sum; i++) {

		macid = (slave_inf_base[i] & MSK_MACID) >> 8;

		tx_buf[i].data[0] = cmd_code | ud_ctr | macid;
		tx_buf[i].data[1] = 0;
		tx_buf[i].data[2] = 0;
		tx_buf[i].data[3] = 0;

#ifdef MASTER_16BIT_ACCESS	/* 16bits bus width */
/* Since an element of data[] is 16bit, there are subscript 4 to 7. */

		tx_buf[i].data[4] = 0;
		tx_buf[i].data[5] = 0;
		tx_buf[i].data[6] = 0;
		tx_buf[i].data[7] = 0;
#endif
	}

/*
* Update Counter and Command Code will be echoed back. As first checking, 
* you should confirm wheather the echoes are received properly.
*/

}


/***************************************************** End of mnm1221_m.c ***/
